// BinTree.  A tree is represented by (a pointer to) its root node.
// The empty tree is represented by null.
// Updating functions (insert, delete) return the updated tree.
// The insert function has the key value as an argument (and not a
// pointer to a node).
// The toString function is intended for test purposes.

import java.util.*;

class BinTree {

    public int key;
    BinTree left;
    BinTree right;
    BinTree p;

    BinTree(int v) {
        left = right = p = null;
        key = v;
    }

    public static BinTree search(BinTree t, int k) {
        if (t == null || k == t.key) return t;
        if (k < t.key) return search(t.left,k);
        else return search(t.right,k);
    }

    public static BinTree insert(BinTree t, int k) {
        BinTree z = new BinTree(k);
        BinTree y = null;
        BinTree x = t;
        while (x != null) {
            y = x;
            if (k < x.key) x = x.left;
            else x = x.right;
	}
        z.p = y;
        if (y == null) return z;
        else {
            if (z.key < y.key) y.left = z;
            else {y.right = z;}
            return t;
	}
    }

    static BinTree minimum(BinTree t) {
        BinTree x = t;
        while (x.left != null) x = x.left;
        return x;
    }

    static BinTree maximum(BinTree t) {
    	  BinTree x = t;
        while (x.right != null) x = x.right;
        return x;
    
    }

    static BinTree successor(BinTree t) {
    	BinTree x = t;
    	if (x.right != null) return minimum(x.right);
    	else { 
    		while ( x.p.right == x) x = x.p; 
    		return x;
    	}
    }

    public static BinTree delete(BinTree t, BinTree z) {
    	BinTree x = null;
    	BinTree y = null;
    	if ( (z.left == null) || (z.right == null) ) y = z;
    		else y = successor(z);
    	if (y.left !=  null) x = y.left;
    		else x = y.right;
    	if (x != null) x.p = y.p;
    	if (y.p == null) t = x;
    	else if (y == y.p.left) y.p.left = x;
    		  else y.p.right = x;
    	if (y != z) z.key = y.key;
    	return y;
    }

    public static String toString(BinTree t) {
        if (t == null) return "";
        if (t.left == null && t.right == null) return "" + t.key;
        return
            "(" + t.key
	    + ((t.left  != null) ? "L" + toString(t.left)  : "") 
            + ((t.right != null) ? "R" + toString(t.right) : "")
	    + ")";
    }

    public static int minDepth(BinTree t) {
    	  if (t==null) return -1; // s den gal!
    	      	  
        if (t.left==null && t.right==null) return 0; // har vi et leaf
        
        if (t.left!=null && t.right==null) return 1+minDepth(t.left);
        if (t.left==null && t.right!=null) return 1+minDepth(t.right);
        
        int a = 1+minDepth(t.left);
        int b = 1+minDepth(t.right);
        return (a<b ? a : b);
    }

    public static int maxDepth(BinTree t) {
        if (t==null) return -1;
        int a = maxDepth(t.left);
        int b = maxDepth(t.right);
        if (a > b) return a+1;
        else return b+1;
    	
    }

    public static void main (String args[]) {
      BinTree t = null; 

		int elements[] = {15,5,16,3,12,20,10,13,18,23,6,7};
		for (int lp1 = 0; lp1 < elements.length; lp1++) {
		 	t = insert(t,elements[lp1]);
		}

	   System.out.println(toString(t));
      System.out.println("MinDepth: " + minDepth(t) );
      System.out.println("MaxDepth: " + maxDepth(t) );

/*      delete(t,search(t,15));
      System.out.println("Deleting 15: " + toString(t) );
      System.out.println("MinDepth: " + minDepth(t) );
      System.out.println("MaxDepth: " + maxDepth(t) );
*/
      delete(t,search(t,3));
      System.out.println("Deleting 3: " + toString(t) );
      System.out.println("MinDepth: " + minDepth(t) );
      System.out.println("MaxDepth: " + maxDepth(t) );
      delete(t,search(t,5));
      System.out.println("Deleting 5: " + toString(t) );
      System.out.println("MinDepth: " + minDepth(t) );
      System.out.println("MaxDepth: " + maxDepth(t) );

		insert(t,8);
		System.out.println("Inserting 8: " + toString(t) );
      System.out.println("MinDepth: " + minDepth(t) );
      System.out.println("MaxDepth: " + maxDepth(t) );
		
      System.out.println("Generating 10 trees of 1001 (Nights-) elements...");
		for (int lp0 = 0; lp0 < 10; lp0++)
		{
			BinTree rnd = null;
			for (int lp1 = 0; lp1 < 1001; lp1++) {
			 	rnd = insert( rnd,(int)(1000*Math.random()) );
			}
			System.out.println("*** New tree: ***");
     		System.out.println("MinDepth: " + minDepth(rnd) );
     		System.out.println("MaxDepth: " + maxDepth(rnd) );
		}

    }
    
}  


