// Enkelt-kdede lister anvendes

class HashTable {

  // Node er et enkelt element i listen.
  class Node {
    Node next = null;
    int key;
    Node(int keyValue) { key = keyValue; }
  }


  // Internt Hast-table
  private Node table[];   


  // Initialiser tabel
  public HashTable(int size) {
  	 table = new Node[size];
  }


  // Hash function
  public int hash(int key) {
  	  return key % table.length; 
  }



  // Inserts a node with key value k if k is not already present 
  // in the table (the node is inserted at the head of the linked 
  // list with index hash(k)).
  // If k was already present in the table then true is returned, 
  // otherwise false is returned. 
  boolean insert(int k) {
    Node cur = table[hash(k)];
    
    while (cur!=null && cur.key!=k) cur = cur.next;
    // cur peger nu p det element, der indeholder k eller null, hvis intet.
    if (cur==null) {
    	cur = table[hash(k)];
    	table[hash(k)] = new Node(k);  // ingen elementer i liste
    	table[hash(k)].next = cur;
    	return false;
    } else 
      return true;
  }
    	

  // Removes a node with key value k from the list
  void delete(int k) {
  	 int hashval = hash(k);
  	 Node cur = table[hashval];
  	 Node last = null;

    while (cur!=null && cur.key!=k) {
    	last = cur;
    	cur = cur.next;
    }
    	
  	 if (cur!=null) {
  	 	if (last==null) table[hashval] = cur.next;
  	 	else last.next = cur.next;
  	 }
  	 
  }

  public String toString() {
  	 String s = "[";
  	 for (int lp1=0; lp1<table.length; lp1++) {
  	 	s += "(";
  	 	Node p = table[lp1];
  	 	while (p!=null) {
  	 		s += p.key+",";
  	 		p = p.next;
  	 	}
  	 	s += "),";
  	 } 
  	 s += "]";
  	 return s;
  }
  	 	

  public static void main (String args[]) {

    int size = 10000;
    int S[] = {5000, 10000, 25000, 50000, 100000}; 

    for (int n = 0; n < S.length; n++) {
      System.out.print("elements= "+S[n]+"\t");
      HashTable t = new HashTable(size); 
    
      int A[] = new int[S[n]];

      // Fill A with random numbers from the interval 0,...,999999
		for (int lp1=0; lp1<A.length; lp1++) 
			A[lp1] = (int) (Math.random()*999999) ;
		
		long t_start, t_end;
		int inserted=0;
		
		t_start = System.currentTimeMillis();
      // Insert the random numbers from A in the hash table
      for (int lp1=0; lp1<A.length; lp1++)
      	if (!t.insert( A[lp1] )) inserted++;
      t_end = System.currentTimeMillis();
      System.out.print("insertion: "+(t_end-t_start)+" ms\tc="+inserted+"\t");
      
      //System.out.println(""+t);

		t_start = System.currentTimeMillis();
      // Delete the random numbers in A from the hash table
      for (int lp1=0; lp1<A.length/2; lp1++)
      	t.delete( A[lp1] );
      t_end = System.currentTimeMillis();
      System.out.println("deletion: "+(t_end-t_start)+" ms");

    	
      //System.out.println(""+t);

   }
   
   
   
  }
}


